from setuptools import setup

setup(name='onstar',
      description='Utility to get data from OnStar service specially from myopel.',
      version='0.1.0',
      url='https://github.com/nyxnyx/onstar',
      author='Grzegorz Szostak',
      author_email='szostak.grzegorz@gmail.com',
      license='MIT',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Information Technology',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3'
      ],
      packages=['onstar']
      )
